@@ -296,6 +296,7 @@ private:
      * Transforms a raw preview image, applying scale and frame.
      *
      * @param pixmap A raw preview image from a PreviewJob.
+     * @param overlays the overlays to add to the pixmap
      * @return The scaled and decorated preview image.
      */
     QPixmap transformPreviewPixmap(const QPixmap &pixmap);
@@ -340,6 +341,7 @@ private:
     void trimHoverSequenceLoadedItems();
 
 private:
+    QSize cacheSize();
     /**
      * enqueue directory size counting for KFileItem item at index
      */
