@@ -7,7 +7,6 @@
 #include "kstandarditemlistwidget.h"
 
 #include "kfileitemlistview.h"
-#include "kfileitemmodel.h"
 #include "private/kfileitemclipboard.h"
 #include "private/kitemlistroleeditor.h"
 #include "private/kitemviewsutils.h"
@@ -18,6 +17,7 @@
 #include <KIconUtils>
 #include <KRatingPainter>
 #include <KStringHandler>
+#include <klocalizedstring.h>
 
 #include <QApplication>
 #include <QGraphicsScene>
@@ -276,7 +276,7 @@ KStandardItemListWidget::KStandardItemListWidget(KItemListWidgetInformant *infor
     , m_expansionArea()
     , m_customTextColor()
     , m_additionalInfoTextColor()
-    , m_overlay()
+    , m_overlays()
     , m_rating()
     , m_roleEditor(nullptr)
     , m_oldRoleEditor(nullptr)
@@ -345,24 +345,39 @@ void KStandardItemListWidget::paint(QPainter *painter, const QStyleOptionGraphic
         drawSiblingsInformation(painter);
     }
 
+    auto pixmap = m_pixmap;
+    if (!m_overlays.isEmpty()) {
+        const qreal dpr = KItemViewsUtils::devicePixelRatio(this);
+
+        const bool iconOnTop = (m_layout == IconsLayout);
+        const KItemListStyleOption &option = styleOption();
+        const qreal padding = option.padding;
+
+        const int widgetIconSize = iconSize();
+        const int maxIconWidth = iconOnTop ? size().width() - 2 * padding : widgetIconSize;
+        const int maxIconHeight = widgetIconSize;
+
+        pixmap = addOverlays(pixmap, m_overlays, QSize(maxIconWidth, maxIconHeight), dpr);
+    }
+
     const KItemListStyleOption &itemListStyleOption = styleOption();
-    if (isHovered() && !m_pixmap.isNull()) {
+    if (isHovered() && !pixmap.isNull()) {
         if (hoverOpacity() < 1.0) {
             /*
-             * Linear interpolation between m_pixmap and m_hoverPixmap.
+             * Linear interpolation between pixmap and m_hoverPixmap.
              *
              * Note that this cannot be achieved by painting m_hoverPixmap over
-             * m_pixmap, even if the opacities are adjusted. For details see
+             * pixmap, even if the opacities are adjusted. For details see
              * https://git.reviewboard.kde.org/r/109614/
              */
-            // Paint pixmap1 so that pixmap1 = m_pixmap * (1.0 - hoverOpacity())
-            QPixmap pixmap1(m_pixmap.size());
-            pixmap1.setDevicePixelRatio(m_pixmap.devicePixelRatio());
+            // Paint pixmap1 so that pixmap1 = pixmap * (1.0 - hoverOpacity())
+            QPixmap pixmap1(pixmap.size());
+            pixmap1.setDevicePixelRatio(pixmap.devicePixelRatio());
             pixmap1.fill(Qt::transparent);
             {
                 QPainter p(&pixmap1);
                 p.setOpacity(1.0 - hoverOpacity());
-                p.drawPixmap(0, 0, m_pixmap);
+                p.drawPixmap(0, 0, pixmap);
             }
 
             // Paint pixmap2 so that pixmap2 = m_hoverPixmap * hoverOpacity()
@@ -376,8 +391,8 @@ void KStandardItemListWidget::paint(QPainter *painter, const QStyleOptionGraphic
             }
 
             // Paint pixmap2 on pixmap1 using CompositionMode_Plus
-            // Now pixmap1 = pixmap2 + m_pixmap * (1.0 - hoverOpacity())
-            //             = m_hoverPixmap * hoverOpacity() + m_pixmap * (1.0 - hoverOpacity())
+            // Now pixmap1 = pixmap2 + pixmap * (1.0 - hoverOpacity())
+            //             = m_hoverPixmap * hoverOpacity() + pixmap * (1.0 - hoverOpacity())
             {
                 QPainter p(&pixmap1);
                 p.setCompositionMode(QPainter::CompositionMode_Plus);
@@ -389,8 +404,8 @@ void KStandardItemListWidget::paint(QPainter *painter, const QStyleOptionGraphic
         } else {
             drawPixmap(painter, m_hoverPixmap);
         }
-    } else if (!m_pixmap.isNull()) {
-        drawPixmap(painter, m_pixmap);
+    } else if (!pixmap.isNull()) {
+        drawPixmap(painter, pixmap);
     }
 
     painter->setFont(m_customizedFont);
@@ -409,15 +424,26 @@ void KStandardItemListWidget::paint(QPainter *painter, const QStyleOptionGraphic
     painter->drawStaticText(textInfo->pos, textInfo->staticText);
 
     bool clipAdditionalInfoBounds = false;
-    if (m_supportsItemExpanding) {
-        // Prevent a possible overlapping of the additional-information texts
-        // with the icon. This can happen if the user has minimized the width
-        // of the name-column to a very small value.
-        const qreal minX = m_pixmapPos.x() + m_pixmap.width() + 4 * itemListStyleOption.padding;
-        if (textInfo->pos.x() + columnWidth("text") > minX) {
-            clipAdditionalInfoBounds = true;
-            painter->save();
-            painter->setClipRect(minX, 0, size().width() - minX, size().height(), Qt::IntersectClip);
+    if (m_supportsItemExpanding && m_sortedVisibleRoles.count() > 1) {
+        // Prevent a possible overlapping of the additional-information-texts with the icon.
+        // This will happen if the user has resized the width of the name-column to be very narrow while having folders expanded.
+        // We only want to draw additional info text outside the area of the icon or expansion area, so we set a clip rect that does not contain the icon area.
+        // This needs to work both for left-to-right as well as right-to-left layout directions.
+        const TextInfo *potentiallyOverlappingRoleText = m_textInfo.value(m_sortedVisibleRoles[1]); // Only the first column after the name column can overlap.
+        if (layoutDirection() == Qt::LeftToRight) { // In left-to-right languages the left end of text would overlap. This is mirrored for right-to-left.
+            const qreal minX = m_iconRect.right() + 2 * itemListStyleOption.padding;
+            if (potentiallyOverlappingRoleText->pos.x() < minX) {
+                clipAdditionalInfoBounds = true;
+                painter->save();
+                painter->setClipRect(minX, 0, size().width() - minX, size().height(), Qt::IntersectClip);
+            }
+        } else {
+            const qreal maxX = m_iconRect.left() - 2 * itemListStyleOption.padding;
+            if (potentiallyOverlappingRoleText->pos.x() + m_customizedFontMetrics.horizontalAdvance(potentiallyOverlappingRoleText->staticText.text()) > maxX) {
+                clipAdditionalInfoBounds = true;
+                painter->save();
+                painter->setClipRect(0, 0, maxX, size().height(), Qt::IntersectClip);
+            }
         }
     }
 
@@ -524,7 +550,11 @@ QRectF KStandardItemListWidget::selectionRect() const
         QRectF adjustedIconRect = iconRect().adjusted(-padding, -padding, padding, padding);
         QRectF result = adjustedIconRect | m_textRect;
         if (m_highlightEntireRow) {
-            result.setRight(m_columnWidthSum + sidePadding());
+            if (layoutDirection() == Qt::LeftToRight) {
+                result.setRight(leftPadding() + m_columnWidthSum);
+            } else {
+                result.setLeft(size().width() - m_columnWidthSum - rightPadding());
+            }
         }
         return result;
     }
@@ -670,7 +700,6 @@ void KStandardItemListWidget::invalidateIconCache()
 {
     m_dirtyContent = true;
     m_dirtyContentRoles.insert("iconPixmap");
-    m_dirtyContentRoles.insert("iconOverlays");
 }
 
 void KStandardItemListWidget::refreshCache()
@@ -722,16 +751,21 @@ QColor KStandardItemListWidget::textColor(const QWidget &widget) const
     return styleOption().palette.color(group, role);
 }
 
-void KStandardItemListWidget::setOverlay(const QPixmap &overlay)
+void KStandardItemListWidget::setOverlays(QHash<Qt::Corner, QString> &overlays)
 {
-    m_overlay = overlay;
+    if (overlays == m_overlays) {
+        return;
+    }
+
+    m_overlays = overlays;
     m_dirtyContent = true;
+    m_dirtyContentRoles.insert("iconOverlays");
     update();
 }
 
-QPixmap KStandardItemListWidget::overlay() const
+QHash<Qt::Corner, QString> KStandardItemListWidget::overlays() const
 {
-    return m_overlay;
+    return m_overlays;
 }
 
 QString KStandardItemListWidget::roleText(const QByteArray &role, const QHash<QByteArray, QVariant> &values) const
@@ -786,9 +820,10 @@ void KStandardItemListWidget::columnWidthChanged(const QByteArray &role, qreal c
     m_dirtyLayout = true;
 }
 
-void KStandardItemListWidget::sidePaddingChanged(qreal padding)
+void KStandardItemListWidget::sidePaddingChanged(qreal leftPaddingWidth, qreal rightPaddingWidth)
 {
-    Q_UNUSED(padding)
+    Q_UNUSED(leftPaddingWidth)
+    Q_UNUSED(rightPaddingWidth)
     m_dirtyLayout = true;
 }
 
@@ -845,16 +880,7 @@ void KStandardItemListWidget::editedRoleChanged(const QByteArray &current, const
     if (current.isEmpty() || !parent || current != "text") {
         if (m_roleEditor) {
             Q_EMIT roleEditingCanceled(index(), current, data().value(current));
-
-            disconnect(m_roleEditor, &KItemListRoleEditor::roleEditingCanceled, this, &KStandardItemListWidget::slotRoleEditingCanceled);
-            disconnect(m_roleEditor, &KItemListRoleEditor::roleEditingFinished, this, &KStandardItemListWidget::slotRoleEditingFinished);
-
-            if (m_oldRoleEditor) {
-                m_oldRoleEditor->deleteLater();
-            }
-            m_oldRoleEditor = m_roleEditor;
-            m_roleEditor->hide();
-            m_roleEditor = nullptr;
+            closeRoleEditor();
         }
         return;
     }
@@ -1021,8 +1047,13 @@ void KStandardItemListWidget::updateExpansionArea()
             const qreal inc = (widgetHeight - widgetIconSize) / 2;
             const qreal x = expandedParentsCount * widgetHeight + inc;
             const qreal y = inc;
-            const qreal xPadding = m_highlightEntireRow ? sidePadding() : 0;
-            m_expansionArea = QRectF(xPadding + x, y, widgetIconSize, widgetIconSize);
+            if (layoutDirection() == Qt::LeftToRight) {
+                const qreal leftPaddingWidth = m_highlightEntireRow ? leftPadding() : 0;
+                m_expansionArea = QRectF(leftPaddingWidth + x, y, widgetIconSize, widgetIconSize);
+                return;
+            }
+            const qreal rightPaddingWidth = m_highlightEntireRow ? rightPadding() : 0;
+            m_expansionArea = QRectF(size().width() - rightPaddingWidth - x - widgetIconSize, y, widgetIconSize, widgetIconSize);
             return;
         }
     }
@@ -1093,14 +1124,15 @@ void KStandardItemListWidget::updatePixmapCache()
             const QStringList overlays = values["iconOverlays"].toStringList();
             const bool hasFocus = scene()->views()[0]->parentWidget()->hasFocus();
             m_pixmap = pixmapForIcon(iconName,
-                                     overlays,
-                                     maxIconHeight,
+                                     QSize(maxIconWidth, maxIconHeight),
                                      m_layout != IconsLayout && isActiveWindow() && isSelected() && hasFocus ? QIcon::Selected : QIcon::Normal);
 
-        } else if (m_pixmap.width() / m_pixmap.devicePixelRatio() != maxIconWidth || m_pixmap.height() / m_pixmap.devicePixelRatio() != maxIconHeight) {
-            // A custom pixmap has been applied. Assure that the pixmap
-            // is scaled to the maximum available size.
-            KPixmapModifier::scale(m_pixmap, QSize(maxIconWidth, maxIconHeight) * dpr);
+        } else {
+            if (m_pixmap.width() / m_pixmap.devicePixelRatio() != maxIconWidth || m_pixmap.height() / m_pixmap.devicePixelRatio() != maxIconHeight) {
+                // A custom pixmap has been applied. Assure that the pixmap
+                // is scaled to the maximum available size.
+                KPixmapModifier::scale(m_pixmap, QSize(maxIconWidth, maxIconHeight) * dpr);
+            }
         }
 
         if (m_pixmap.isNull()) {
@@ -1109,14 +1141,7 @@ void KStandardItemListWidget::updatePixmapCache()
         }
 
         if (m_isCut) {
-#if KICONTHEMES_VERSION >= QT_VERSION_CHECK(6, 5, 0)
             KIconEffect::toDisabled(m_pixmap);
-#else
-            QImage img = m_pixmap.toImage();
-            KIconEffect::toGray(img, 1);
-            KIconEffect::semiTransparent(img);
-            m_pixmap = QPixmap::fromImage(img);
-#endif
         }
 
         if (m_isHidden) {
@@ -1135,11 +1160,6 @@ void KStandardItemListWidget::updatePixmapCache()
         }
     }
 
-    if (!m_overlay.isNull()) {
-        QPainter painter(&m_pixmap);
-        painter.drawPixmap(0, (m_pixmap.height() - m_overlay.height()) / m_pixmap.devicePixelRatio(), m_overlay);
-    }
-
     int scaledIconSize = 0;
     if (iconOnTop) {
         const TextInfo *textInfo = m_textInfo.value("text");
@@ -1164,8 +1184,8 @@ void KStandardItemListWidget::updatePixmapCache()
     } else {
         // Center horizontally and vertically within the icon-area
         const TextInfo *textInfo = m_textInfo.value("text");
-        if (QApplication::isRightToLeft() && m_layout == CompactLayout) {
-            m_pixmapPos.setX(size().width() - padding - (scaledIconSize + m_scaledPixmapSize.width()) / 2.0);
+        if (QApplication::isRightToLeft()) {
+            m_pixmapPos.setX(m_textRect.right() + 2.0 * padding);
         } else {
             m_pixmapPos.setX(textInfo->pos.x() - 2.0 * padding - (scaledIconSize + m_scaledPixmapSize.width()) / 2.0);
         }
@@ -1189,13 +1209,7 @@ void KStandardItemListWidget::updatePixmapCache()
     // Prepare the pixmap that is used when the item gets hovered
     if (isHovered()) {
         m_hoverPixmap = m_pixmap;
-#if KICONTHEMES_VERSION >= QT_VERSION_CHECK(6, 5, 0)
         KIconEffect::toActive(m_hoverPixmap);
-#else
-        QImage img = m_pixmap.toImage();
-        KIconEffect::toGamma(img, 0.7);
-        m_hoverPixmap = QPixmap::fromImage(img);
-#endif
     } else if (hoverOpacity() <= 0.0) {
         // No hover animation is ongoing. Clear m_hoverPixmap to save memory.
         m_hoverPixmap = QPixmap();
@@ -1273,23 +1287,6 @@ void KStandardItemListWidget::updateTextsCache()
     }
 }
 
-QString KStandardItemListWidget::elideRightKeepExtension(const QString &text, int elidingWidth) const
-{
-    const auto extensionIndex = text.lastIndexOf('.');
-    if (extensionIndex != -1) {
-        // has file extension
-        const auto extensionLength = text.length() - extensionIndex;
-        const auto extensionWidth = m_customizedFontMetrics.horizontalAdvance(text.right(extensionLength));
-        if (elidingWidth > extensionWidth && extensionLength < 6 && (float(extensionWidth) / float(elidingWidth)) < 0.3) {
-            // if we have room to display the file extension and the extension is not too long
-            QString ret = m_customizedFontMetrics.elidedText(text.chopped(extensionLength), Qt::ElideRight, elidingWidth - extensionWidth);
-            ret.append(QStringView(text).right(extensionLength));
-            return ret;
-        }
-    }
-    return m_customizedFontMetrics.elidedText(text, Qt::ElideRight, elidingWidth);
-}
-
 QString KStandardItemListWidget::escapeString(const QString &text) const
 {
     QString escaped(text);
@@ -1350,7 +1347,7 @@ void KStandardItemListWidget::updateIconsLayoutTextCache()
                 qreal lastLineWidth;
                 do {
                     QString lastTextLine = nameText.mid(line.textStart());
-                    lastTextLine = elideRightKeepExtension(lastTextLine, elidingWidth);
+                    lastTextLine = m_customizedFontMetrics.elidedText(lastTextLine, Qt::ElideMiddle, elidingWidth);
                     const QString elidedText = nameText.left(line.textStart()) + lastTextLine;
                     nameTextInfo->staticText.setText(elidedText);
 
@@ -1398,7 +1395,7 @@ void KStandardItemListWidget::updateIconsLayoutTextCache()
             textLine.setLineWidth(maxWidth);
             requiredWidth = textLine.naturalTextWidth();
             if (requiredWidth > maxWidth) {
-                const QString elidedText = elideRightKeepExtension(text, maxWidth);
+                const QString elidedText = m_customizedFontMetrics.elidedText(text, Qt::ElideMiddle, maxWidth);
                 textInfo->staticText.setText(elidedText);
                 requiredWidth = m_customizedFontMetrics.horizontalAdvance(elidedText);
             } else if (role == "rating") {
@@ -1450,7 +1447,7 @@ void KStandardItemListWidget::updateCompactLayoutTextCache()
         qreal requiredWidth = m_customizedFontMetrics.horizontalAdvance(text);
         if (requiredWidth > maxWidth) {
             requiredWidth = maxWidth;
-            const QString elidedText = elideRightKeepExtension(text, maxWidth);
+            const QString elidedText = m_customizedFontMetrics.elidedText(text, Qt::ElideMiddle, maxWidth);
             textInfo->staticText.setText(elidedText);
         }
 
@@ -1473,6 +1470,8 @@ void KStandardItemListWidget::updateDetailsLayoutTextCache()
     // +------+
     // | Icon |  Name role   Additional role 1   Additional role 2
     // +------+
+    // Mirror the above for right-to-left languages.
+    const bool isLeftToRight = QApplication::layoutDirection() == Qt::LeftToRight;
     m_textRect = QRectF();
 
     const KItemListStyleOption &option = styleOption();
@@ -1484,9 +1483,10 @@ void KStandardItemListWidget::updateDetailsLayoutTextCache()
     const qreal columnWidthInc = columnPadding(option);
     qreal firstColumnInc = iconSize();
     if (m_supportsItemExpanding) {
-        firstColumnInc += (m_expansionArea.left() + m_expansionArea.right() + widgetHeight) / 2;
+        firstColumnInc += isLeftToRight ? (m_expansionArea.left() + m_expansionArea.right() + widgetHeight) / 2
+                                        : ((size().width() - m_expansionArea.left()) + (size().width() - m_expansionArea.right()) + widgetHeight) / 2;
     } else {
-        firstColumnInc += option.padding + sidePadding();
+        firstColumnInc += option.padding + (isLeftToRight ? leftPadding() : rightPadding());
     }
 
     qreal x = firstColumnInc;
@@ -1503,27 +1503,26 @@ void KStandardItemListWidget::updateDetailsLayoutTextCache()
         const bool isTextRole = (role == "text");
         if (isTextRole) {
             text = escapeString(text);
-            availableTextWidth -= firstColumnInc - sidePadding();
+            availableTextWidth -= firstColumnInc - (isLeftToRight ? leftPadding() : rightPadding());
         }
 
         if (requiredWidth > availableTextWidth) {
-            text = elideRightKeepExtension(text, availableTextWidth);
+            text = m_customizedFontMetrics.elidedText(text, Qt::ElideMiddle, availableTextWidth);
             requiredWidth = m_customizedFontMetrics.horizontalAdvance(text);
         }
 
         TextInfo *textInfo = m_textInfo.value(role);
         textInfo->staticText.setText(text);
-        textInfo->pos = QPointF(x + columnWidthInc / 2, y);
+        textInfo->pos = QPointF(isLeftToRight ? (x + columnWidthInc / 2) : (size().width() - (x + columnWidthInc / 2) - requiredWidth), y);
         x += roleWidth;
 
         if (isTextRole) {
-            const qreal textWidth = option.extendedSelectionRegion ? size().width() - textInfo->pos.x() : requiredWidth + 2 * option.padding;
-            m_textRect = QRectF(textInfo->pos.x() - option.padding, 0, textWidth, size().height());
+            m_textRect = QRectF(textInfo->pos.x() - option.padding, 0, requiredWidth + 2 * option.padding, size().height());
 
             // The column after the name should always be aligned on the same x-position independent
             // from the expansion-level shown in the name column
-            x -= firstColumnInc - sidePadding();
-        } else if (isRoleRightAligned(role)) {
+            x -= firstColumnInc - (isLeftToRight ? leftPadding() : rightPadding());
+        } else if (isRoleRightAligned(role) && isLeftToRight) {
             textInfo->pos.rx() += roleWidth - requiredWidth - columnWidthInc;
         }
     }
@@ -1554,6 +1553,76 @@ void KStandardItemListWidget::updateAdditionalInfoTextColor()
         QColor((c1.red() * p1 + c2.red() * p2) / 100, (c1.green() * p1 + c2.green() * p2) / 100, (c1.blue() * p1 + c2.blue() * p2) / 100);
 }
 
+QPixmap
+KStandardItemListWidget::addOverlays(const QPixmap &pixmap, const QHash<Qt::Corner, QString> &overlays, const QSize &size, qreal dpr, QIcon::Mode mode) const
+{
+    // similar to KIconUtils::addOverlays, keep in sync preferrably
+    if (overlays.isEmpty()) {
+        return pixmap;
+    }
+
+    int width = size.width();
+    int height = size.height();
+    const int iconSize = qMin(width, height);
+
+    // Determine the overlay icon
+    int overlaySize;
+    if (iconSize < 32) {
+        overlaySize = 8;
+    } else if (iconSize <= 48) {
+        overlaySize = 16;
+    } else if (iconSize <= 96) {
+        overlaySize = 22;
+    } else if (iconSize < 256) {
+        overlaySize = 32;
+    } else {
+        overlaySize = 64;
+    }
+
+    auto phyiscalSize = QSize(std::clamp(pixmap.width(), qFloor(2 * overlaySize * dpr), qFloor(size.width() * dpr)),
+                              std::clamp(pixmap.height(), qFloor(2 * overlaySize * dpr), qFloor(size.height() * dpr)));
+
+    QPixmap output(phyiscalSize);
+    output.setDevicePixelRatio(dpr);
+    output.fill(Qt::transparent);
+
+    QPainter painter(&output);
+    painter.drawPixmap(qFloor(phyiscalSize.width() / dpr / 2) - qFloor(pixmap.width() / pixmap.devicePixelRatio() / 2),
+                       // align the icon to the bottom to match the behavior elsewhere
+                       qFloor(phyiscalSize.height() / dpr) - qFloor(pixmap.height() / pixmap.devicePixelRatio()),
+                       pixmap);
+
+    width = qCeil(phyiscalSize.width() / dpr);
+    height = qCeil(phyiscalSize.height() / dpr);
+
+    // Iterate over stored overlays
+    for (const auto &[corner, overlay] : overlays.asKeyValueRange()) {
+        const QPixmap overlayPixmap = QIcon::fromTheme(overlay).pixmap(QSize{overlaySize, overlaySize}, dpr, mode);
+        if (overlayPixmap.isNull()) {
+            continue;
+        }
+
+        QPoint startPoint;
+        switch (corner) {
+        case Qt::BottomLeftCorner:
+            startPoint = QPoint{0, height - overlaySize};
+            break;
+        case Qt::BottomRightCorner:
+            startPoint = QPoint{width - overlaySize, height - overlaySize};
+            break;
+        case Qt::TopRightCorner:
+            startPoint = QPoint{width - overlaySize, 0};
+            break;
+        case Qt::TopLeftCorner:
+            startPoint = QPoint{};
+            break;
+        }
+        painter.drawPixmap(startPoint, overlayPixmap);
+    }
+
+    return output;
+}
+
 void KStandardItemListWidget::drawPixmap(QPainter *painter, const QPixmap &pixmap)
 {
     if (m_scaledPixmapSize != pixmap.size() / pixmap.devicePixelRatio()) {
@@ -1603,7 +1672,7 @@ void KStandardItemListWidget::drawSiblingsInformation(QPainter *painter)
 
         style()->drawPrimitive(QStyle::PE_IndicatorBranch, &option, painter);
 
-        siblingRect.translate(-siblingRect.width(), 0);
+        siblingRect.translate(layoutDirection() == Qt::LeftToRight ? -siblingRect.width() : siblingRect.width(), 0);
     }
 }
 
@@ -1641,15 +1710,15 @@ void KStandardItemListWidget::closeRoleEditor()
     m_roleEditor = nullptr;
 }
 
-QPixmap KStandardItemListWidget::pixmapForIcon(const QString &name, const QStringList &overlays, int size, QIcon::Mode mode) const
+QPixmap KStandardItemListWidget::pixmapForIcon(const QString &name, const QSize &size, QIcon::Mode mode) const
 {
     static const QIcon fallbackIcon = QIcon::fromTheme(QStringLiteral("unknown"));
     const qreal dpr = KItemViewsUtils::devicePixelRatio(this);
 
-    size *= dpr;
+    int iconHeight = size.height();
+    QSize iconSize = QSize(iconHeight, iconHeight);
 
-    const QString key = "KStandardItemListWidget:" % name % ":" % overlays.join(QLatin1Char(':')) % ":" % QString::number(size) % "@" % QString::number(dpr)
-        % ":" % QString::number(mode);
+    const QString key = "KStandardItemListWidget:" % name % ":" % QString::number(iconHeight) % "@" % QString::number(dpr) % ":" % QString::number(mode);
     QPixmap pixmap;
 
     if (!QPixmapCache::find(key, &pixmap)) {
@@ -1657,25 +1726,15 @@ QPixmap KStandardItemListWidget::pixmapForIcon(const QString &name, const QStrin
         if (icon.isNull()) {
             icon = QIcon(name);
         }
-        if (icon.isNull() || icon.pixmap(size / dpr, size / dpr, mode).isNull()) {
+        if (!icon.isNull()) {
+            pixmap = icon.pixmap(iconSize, dpr, mode);
+        }
+        if (pixmap.isNull()) {
             icon = fallbackIcon;
+            pixmap = icon.pixmap(iconSize, dpr, mode);
         }
-
-        pixmap = icon.pixmap(QSize(size / dpr, size / dpr), dpr, mode);
-        if (pixmap.width() != size || pixmap.height() != size) {
-            KPixmapModifier::scale(pixmap, QSize(size, size));
-        }
-
-        // Strangely KFileItem::overlays() returns empty string-values, so
-        // we need to check first whether an overlay must be drawn at all.
-        for (const QString &overlay : overlays) {
-            if (!overlay.isEmpty()) {
-                // There is at least one overlay, draw all overlays above m_pixmap
-                // and cancel the check
-                const QSize size = pixmap.size();
-                pixmap = KIconUtils::addOverlays(pixmap, overlays).pixmap(size, dpr, mode);
-                break;
-            }
+        if (pixmap.width() != iconHeight * dpr || pixmap.height() != iconHeight * dpr) {
+            KPixmapModifier::scale(pixmap, iconSize * dpr);
         }
 
         QPixmapCache::insert(key, pixmap);
